from django.contrib.auth.mixins import UserPassesTestMixin
from django.urls import reverse


from aircox.forms import EpisodeForm
from ..filters import EpisodeFilters
from ..models import Episode, Program, StaticPage
from .page import PageListView
from .program import ProgramPageDetailView, BaseProgramMixin
from .page import PageUpdateView


__all__ = (
    "EpisodeDetailView",
    "EpisodeListView",
    "PodcastListView",
    "EpisodeUpdateView",
)


class EpisodeDetailView(ProgramPageDetailView):
    model = Episode

    def get_context_data(self, **kwargs):
        if "tracks" not in kwargs:
            kwargs["tracks"] = self.object.track_set.order_by("position")
        return super().get_context_data(**kwargs)

    def get_related_queryset(self):
        return (
            self.get_queryset().parent(self.object.parent).exclude(pk=self.object.pk).published().order_by("-pub_date")
        )

    def get_related_url(self):
        return reverse("episode-list", kwargs={"parent_slug": self.object.parent.slug})


class EpisodeListView(PageListView):
    model = Episode
    filterset_class = EpisodeFilters
    parent_model = Program
    attach_to_value = StaticPage.Target.EPISODES


class PodcastListView(EpisodeListView):
    attach_to_value = StaticPage.Target.PODCASTS
    queryset = Episode.objects.published().with_podcasts().order_by("-pub_date")


class EpisodeUpdateView(UserPassesTestMixin, BaseProgramMixin, PageUpdateView):
    model = Episode
    form_class = EpisodeForm

    def get_sidebar_queryset(self):
        return super().get_sidebar_queryset().filter(parent=self.program)

    def test_func(self):
        program = self.get_object().program
        return self.request.user.has_perm("aircox.%s" % program.change_permission_codename)

    def get_success_url(self):
        return reverse("episode-detail", kwargs={"slug": self.get_object().slug})
